<?php
// ####################################################################
// Version: 0.3.0
// Autor: Markus Widmer
// Erstellungsdatum: 11.10.2006
// Letzte Aenderung: 15.02.2008



class Node implements iNode {

  // ## Klassenvariablen ##############################################
  private $value = false;
  private $name = "NODE";
  private $child = Array();
  private $attribute = Array();




  // ## Konstruktor ###################################################
  public function __construct() {
  }




  // ## Destruktor ####################################################
  public function __destruct() {
  }




  // ## setName #######################################################
  public function setName( $inName ) {

    // Der Name darf weder ein Objekt, ein Array oder eine
    // ausfuehrbare Funktion sein, um gespeichert zu werden.
    if(    !is_object( $inName )
        && !is_array( $inName ) ) {

      $this->name = strtolower( $inName );


      return true;

    }
    else {

      return false;

    }

  }




  // ## getName #######################################################
  public function getName() {

    return $this->name;

  }




  // ## setAttribute ##################################################
  public function setAttribute( $inName, $inValue ) {

    // Der Name darf weder ein Objekt, ein Array oder eine
    // ausfuehrbare Funktion sein. Ebenso der Wert.
    if(    !is_object( $inName )
        && !is_array( $inName )
        && !is_object( $inValue )
        && !is_array( $inValue ) ) {

      $this->attribute[$inName] = strval( $inValue );


      return true;

    }
    else {

      return false;

    }

  }




  // ## getAttribute ##################################################
  public function getAttribute( $inName ) {

    // Das Attribut muss einen Wert haben
    if( isset( $this->attribute[$inName] ) ) {

      return $this->attribute[$inName];

    }

    // Sonst wird ein leerer String zurueckgegeben
    else {

      return "";

    }

  }



  // ## setValue ######################################################
  public function setValue( $inValue ) {

    // Ein Knoten kann nur dann einen Wert haben, wenn
    // er keine Kinderknoten hat.
    if( sizeof( $this->child ) == 0 ) {

      // Der Wert darf weder ein Objekt, ein Array oder eine
      // ausfuehrbare Funktion sein, um gespeichert zu werden.
      if(    !is_object( $inValue )
          && !is_array( $inValue ) ) {

        $this->value = strval( $inValue );


        return true;

      }
      else {

        return false;

      }

    }
    else {

      return false;

    }

  }



  // ## getValue ######################################################
  public function getValue() {

    return $this->value;

  }



  // ## addChild ######################################################
  public function addChild( iNode $inNode ) {

    // Ein Knoten kann nur dann Kinderknoten haben, wenn er
    // nicht bereits einen Wert hat. Der uebergebene Wert
    // muss zudem auch wirklich ein Knoten sein.
    if(    ($this->value == false)
        && ($inNode instanceof iNode) ) {

      $this->child[] = $inNode;


      return true;

    }
    else {

      return false;

    }

  }




  // ## getChildArray #################################################
  public function getChildArray() {

    if( is_array( $this->child ) ) {

      return $this->child;

    }
    else {

      return Array();

    }

  }




  // ## getChild ######################################################
  public function getChild( $inName, $inNumber ) {

    // Es wird auf jeden Fall ein korrekter Knoten
    // zurueckgegeben, auch wenn keiner gefunden
    // wurde.
    $wantedNode = new Node();
    $wantedNode->setName( $inName );


    // Die Suche nach dem Knoten beginnen und alle
    // Kinderknoten ansehen.
    $i = 0;
    $n = 0;
    $flagFound = false;
    while(    ($i < sizeof( $this->child ))
           && !$flagFound ) {

      // Einen Knoten holen
      $child = $this->child[$i];


      // Name ist korrekt...
      if( preg_match( "/^" . $child->getName() . "$/i", $inName ) ) {

        // Nummer ist korrekt...
        if( $inNumber == $n ) {

          $flagFound = true;
          $wantedNode = $child;

        }

        // Nummer ist nicht korrekt...
        else {

          $n++;

        }

      }


      // Schleifenvariable hochzaehlen
      $i++;

    }


    return $wantedNode;

  }




  // ## searchChild ###################################################
  public function searchChild( $inName, $inAttribute, $inRegex ) {

    $arrNode = Array();   // Der Rueckgabewert
    $child = new Node();  // Temporaerer Knoten
    $i = 0;               // Schleifenvariable


    for( $i = 0; $i < $this->countChilds( $inName ); $i++ ) {

      $child = $this->getChild( $inName, $i );

      if( preg_match( $inRegex, $child->getAttribute( $inAttribute ) ) ) {

        $arrNode[] = $child;

      }

    }


    return $arrNode;

  }




  // ## countChilds ###################################################
  public function countChilds( $inName ) {

    $number = 0;  // Anzahl der Knoten
    $i = 0;       // Schleifenvariable


    // Alle Knoten durchgehen
    for( $i = 0; $i < sizeof( $this->child ); $i++ ) {

      if( preg_match( "/^" . $this->child[$i]->getName() . "$/i", $inName ) ) {

        $number++;

      }

    }


    return $number;

  }

}
?>